<?php
if (!defined('ABSPATH')) {
    exit;
}

function api_post_grid_get_posts(WP_REST_Request $request)
{
    
    $post_type = sanitize_text_field($request->get_param('post_type')) ?: 'post';
    $taxonomy = sanitize_text_field($request->get_param('taxonomy'));
    $term_id = absint($request->get_param('term_id'));
    $category = absint($request->get_param('category'));
    $per_page = absint($request->get_param('per_page')) ?: 12;
    $page = absint($request->get_param('page')) ?: 1;
    $order = in_array(strtoupper($request->get_param('order')), ['ASC', 'DESC']) ? strtoupper($request->get_param('order')) : 'DESC';
    $orderby = sanitize_text_field($request->get_param('orderby')) ?: 'date';

    error_log('API Post Grid - Received parameters: ' . print_r([
        'post_type' => $post_type,
        'taxonomy' => $taxonomy,
        'term_id' => $term_id,
        'category' => $category,
        'per_page' => $per_page,
        'page' => $page,
        'order' => $order,
        'orderby' => $orderby
    ], true));

    try {
        $args = [
            'post_type' => $post_type,
            'posts_per_page' => $per_page,
            'paged' => $page,
            'order' => $order,
            'orderby' => $orderby,
            'post_status' => 'publish'
        ];

        $tax_queries = [];

        if (!empty($taxonomy) && !empty($term_id)) {
            $tax_queries[] = [
                'taxonomy' => $taxonomy,
                'field' => 'term_id',
                'terms' => $term_id,
                'include_children' => true
            ];
        }

        if (!empty($category)) {
            $tax_queries[] = [
                'taxonomy' => 'category',
                'field' => 'term_id',
                'terms' => $category,
                'include_children' => true
            ];
        }

        if (!empty($tax_queries)) {
            if (count($tax_queries) > 1) {
                $args['tax_query'] = $tax_queries;
                $args['tax_query']['relation'] = 'AND';
            } else {
                $args['tax_query'] = $tax_queries;
            }
        }

        error_log('API Post Grid - WP_Query arguments: ' . print_r($args, true));

        $query = new WP_Query($args);
        $posts = [];

        if ($query->have_posts()) {
            while ($query->have_posts()) {
                $query->the_post();

                $categories = [];
                $post_categories = get_the_category();
                if (!empty($post_categories)) {
                    foreach ($post_categories as $cat) {
                        $categories[] = [
                            'id' => $cat->term_id,
                            'name' => $cat->name,
                            'slug' => $cat->slug,
                        ];
                    }
                }

                $image_full = get_the_post_thumbnail_url(get_the_ID(), 'full');
                $image_thumb = get_the_post_thumbnail_url(get_the_ID(), 'medium');

                if (!$image_full) {
                    $image_full = plugin_dir_url(dirname(__FILE__)) . 'assets/images/placeholder.jpg';
                }
                if (!$image_thumb) {
                    $image_thumb = plugin_dir_url(dirname(__FILE__)) . 'assets/images/placeholder.jpg';
                }
                
                $posts[] = [
                    'id' => get_the_ID(),
                    'title' => get_the_title(),
                    'link' => get_permalink(),
                    'image' => $image_full,
                    'image_thumbnail' => $image_thumb,
                    'excerpt' => get_the_excerpt(),
                    'categories' => $categories
                ];
            }
        }

        wp_reset_postdata();

        error_log('API Post Grid - Found Posts: ' . count($posts));

        return rest_ensure_response([
            'success' => true,
            'data' => [
                'posts' => $posts,
                'total' => (int) $query->found_posts,
                'max_pages' => (int) $query->max_num_pages,
                'current_page' => (int) $page,
                'per_page' => (int) $per_page
            ]
        ]);
    } catch (Exception $e) {
        error_log('API Post Grid - Error: ' . $e->getMessage());
        
        return rest_ensure_response([
            'success' => false,
            'message' => $e->getMessage(),
            'data' => [
                'posts' => [],
                'total' => 0,
                'max_pages' => 0,
                'current_page' => (int) $page,
                'per_page' => (int) $per_page
            ]
        ]);
    }
}

function api_post_grid_get_post_types()
{
    $post_types = get_post_types(['public' => true], 'objects');
    $data = [];

    foreach ($post_types as $post_type) {
        
        if (!in_array($post_type->name, ['attachment', 'revision', 'nav_menu_item'])) {
            $data[] = [
                'value' => $post_type->name,
                'label' => $post_type->label
            ];
        }
    }

    return rest_ensure_response([
        'success' => true,
        'data' => $data
    ]);
}

function api_post_grid_get_taxonomies($request)
{
    $post_type = sanitize_text_field($request['post_type']);

    if (!post_type_exists($post_type)) {
        return new WP_Error(
            'invalid_post_type',
            'Invalid post type specified',
            array('status' => 404)
        );
    }

    $taxonomies = get_object_taxonomies($post_type, 'objects');
    $data = [];

    foreach ($taxonomies as $taxonomy) {
        
        if ($taxonomy->public && $taxonomy->publicly_queryable) {
            $data[] = [
                'value' => $taxonomy->name,
                'label' => $taxonomy->labels->singular_name 
            ];
        }
    }

    return rest_ensure_response([
        'success' => true,
        'data' => $data
    ]);
}

function api_post_grid_get_terms($request)
{
    $taxonomy = sanitize_text_field($request['taxonomy']);

    if (!taxonomy_exists($taxonomy)) {
        return new WP_Error(
            'invalid_taxonomy',
            'Invalid taxonomy specified',
            array('status' => 404)
        );
    }

    $args = [
        'taxonomy' => $taxonomy,
        'hide_empty' => false,
        'orderby' => 'name',
        'order' => 'ASC'
    ];

    $terms = get_terms($args);

    if (is_wp_error($terms)) {
        return rest_ensure_response([
            'success' => false,
            'message' => $terms->get_error_message()
        ]);
    }

    $data = [];
    foreach ($terms as $term) {
        $data[] = [
            'value' => $term->term_id, 
            'label' => $term->name,
            'count' => $term->count 
        ];
    }

    return rest_ensure_response([
        'success' => true,
        'data' => $data
    ]);
}